package ThumbGD;
use strict;
use warnings;
use Carp;

sub new {
  my $this = shift;
  my $class = ref($this) || $this;
  my $self = {};
  bless $self, $class;
}

sub destroy {
  $_[0]->{img} = undef;
}

sub setsize {
  my ($self, $x, $y) = @_;
  $self->{img} = GD::Image->new($x || 64, $y || 64, 1); # you have to define dimensions now (or you get a 64x64 image anyway)
}

sub create {
  my ($self, $out) = @_;
  open(my $fh, '>', $out) or croak "Unable to write $out";
  binmode $fh;
  print $fh $self->{img}->png;
  close $fh;
}

sub setcolour {
  my ($self, $value) = @_;
  my @a = map { hex $_ } $value =~ /(..)/g;
  return $self->{img}->colorAllocate(@a);
}

sub stone {
  my ($self, $sf, $st, $outline, $x, $y, $fgcolour, $bgcolour) = @_;
  my $st95 = $sf * $st * 1.03; # stonefactor was originally harcoded as 0.95, hence the name
  my $simg = $self->{img};
  $simg->filledEllipse($x, $y, $st95, $st95, $bgcolour);
  if ($outline) {
    $simg->setThickness($st/20);
    $simg->ellipse($x, $y, $st95, $st95, $fgcolour);
    $simg->setThickness(1);
  }
}

sub stonef {
  my ($self, $x, $y, $colour, $factor, $st, $fb, $path) = @_;
  $self->{img}->stringFT($colour, $path, $factor, 0, $x - $st/2, $y + $st/2, chr($fb));
}

sub dots {
  my ($self, $st, $x, $y, $linecolour) = @_;
  $self->{img}->filledEllipse($x, $y, $st/10 + 2, $st/10 + 2, $linecolour);
}

sub board {
  my ($self, $writer, $x, $y, $bo, @p) = @_;
  my $obcolour = $writer->{colours}->{offboard};
  $self->{img}->filledRectangle(0, 0, $x, $bo, $obcolour);
  $self->{img}->filledRectangle(0, 0, $bo, $y, $obcolour);
  $self->{img}->filledRectangle($x - $bo, 0, $x, $y, $obcolour);
  $self->{img}->filledRectangle(0, $y - $bo, $x, $y, $obcolour);
  $self->{img}->filledRectangle(@p, $writer->{colours}->{board});
}

sub square {
  my ($self, $colour, @p) = @_;
  $self->{img}->filledRectangle(@p, $colour);
}

sub rotate {
  my ($self) = @_;
  $self->{img}->copyRotate180();
}

sub line {
  my ($self, $colour, $halfstone, @p) = @_;
  $self->{img}->setThickness($halfstone/20);
  $self->{img}->line(@p, $colour);
  $self->{img}->setThickness(1);  
}

sub crop {
  my ($self, $geowidth, $geoheight, $minx, $miny) = @_;
  my $newimg = GD::Image->new($geowidth, $geoheight, 1);
  $newimg->copy($self->{img}, 0, 0, $minx, $miny, $geowidth, $geoheight);
  $self->{img} = $newimg;
}

sub frame {
  my ($self, $writer, $x, $y) = @_;
  my $extension = 2 * $writer->{extension} * $writer->{stonesize};
  my $newimg = GD::Image->new($x + $extension, $y + $extension, 1);
  my $obcolour = $writer->{colours}->{offboard};
  $newimg->filledRectangle(0, 0, $x + $extension, $extension/2, $obcolour);
  $newimg->filledRectangle(0, 0, $extension/2, $y + $extension, $obcolour);
  $newimg->filledRectangle($x + $extension/2, 0, $x + $extension, $y + $extension, $obcolour);
  $newimg->filledRectangle(0, $y + $extension/2, $x + $extension, $y + $extension, $obcolour);
  $newimg->copy($self->{img}, $extension/2, $extension/2, 0, 0, $x, $y);
  $self->{img} = $newimg;
}

sub resize {
  my ($self, $scale) = @_;
  my $simg = $self->{img};
  my ($a, $b) = dimension($simg->width, $simg->height, $scale);
  my $newimg2 = GD::Image->new($a, $b, 1);
  $newimg2->copyResampled($simg, 0, 0, 0, 0, $a, $b, $simg->width, $simg->height);
  $self->{img} = $newimg2;
}

sub dimension {
  my ($width, $height, $scale) = @_;
  if ($width > $height) {
    $height *= $scale/$width;
    $width = $scale;
  } else {
    $width *= $scale/$height;
    $height = $scale;
  }
  return int($width + 0.5), int($height + 0.5)
}

1;
